/* H8_LAN sample program */

#include <h8/reg3067.h>
#include <h8/syscall.h>
#include <string.h>
#include "h8_lan.h"

#define DEBUG_COMMAND	0
#define DEBUG_PRINT		0

#if DEBUG_PRINT
#define		DbgPrint(_x_)	MyPrint(_x_)
#else
#define		DbgPrint(_x_)
#endif


void MyPrint(char *pString)
{
	sio_change(1);
	write_mode(ASCII);
	write_string(pString);
	write_string("\n");
}


void MyWait(unsigned int dwWait)
{
	unsigned int i;
	for (i=0; i<dwWait; i++) {
		;
	}
}

/********************************************************************************/

void SetBaudrate(int rate)
{
	g_Baudrate = rate;
}

int GetBaudrate(void)
{
	return g_Baudrate;
}


int GetInitState(void)
{
	return ImageInitial;
}

int GetCameraState(void)
{
	return g_CameraState;
}

void IncrementDataCount(void)
{
	imatge_data_count++;
}

unsigned long GetTotalImageSize(void)
{
	return imatge_data_length;
}

//////////////////////////////////////////////////
//
// VAJɃR}h𑗐M
//
//////////////////////////////////////////////////
unsigned char Send_Cmd (
		unsigned char cmd, 
		unsigned char param1, 
		unsigned char param2, 
		unsigned char param3, 
		unsigned char param4)
{
	unsigned char	bRet = C1098_FALSE;
	int				iLen;
	char			*pBuf = NULL;
	
	SendCmd.Sync	= 0xAA;
	SendCmd.CmdID	= cmd;
	SendCmd.Param1	= param1;
	SendCmd.Param2	= param2;
	SendCmd.Param3	= param3;
	SendCmd.Param4	= param4;

	iLen = sizeof(comedia_C1098_CMD);
	pBuf = (char *)&SendCmd;

	sio_change(0);
	write_mode(SIO);
	if (sys_write(pBuf, iLen) == 0) {
		DbgPrint("write done");
		bRet = C1098_TRUE;
	} else {
		DbgPrint("write fail");
	}
	return bRet;
}

//////////////////////////////////////////////////
//
// VAJR}hM
//
//////////////////////////////////////////////////
unsigned char Recv_Cmd (
			char *buf, 
			long length)
{
	unsigned char bRet = C1098_FALSE;

	
	sio_change(0);
	read_mode(SIO);

	if (sys_read(buf, length) != 0) {
		DbgPrint("read done");
		bRet = C1098_TRUE;
	} else {
		//DbgPrint("read fail");
	}
	return bRet;
}


//////////////////////////////////////////////////
//
// VAJɃZbgR}h𑗐M
//
//////////////////////////////////////////////////
unsigned char Send_Reset_Cmd( void )
{
	unsigned char bRet = C1098_FALSE;
	
	while (bRet == C1098_FALSE) {
		if (imagereset == 100) {
			break;
		}
		if (Send_Cmd (C1098_CMD_RESET, 0x00, 0x00, 0x00, 0xff) == C1098_TRUE) {
			if (Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD)) == C1098_TRUE) {
				if (RcvCmd.CmdID == C1098_CMD_ACK) {
					bRet = C1098_TRUE;
				} else {
					imagereset++;
				}
			} else {
				imagereset++;
			}
		} else {
			imagereset++;
		}
	}
	return bRet;
}

//////////////////////////////////////////////////
//
// VAJɓR}h𑗐M
//
//////////////////////////////////////////////////
unsigned char Send_SYNC_Cmd (void)
{
	unsigned char bRet = C1098_FALSE;
	int i = 0;
	
	// SYNC R}h𑗐Mꍇ́A
	while (1) {
		i++;
		if(i > 60) {
			bRet = C1098_FALSE;
			DbgPrint("timeout!!");
			break;
		}
		
		Send_Cmd (C1098_CMD_SYNC, 0x00, 0x00, 0x00, 0x00);
		g_CameraState = SYNC_SEND;

		MyWait(0xFFFFF);
		
		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		bRet = Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD));
		DbgPrint("recv done");

		if ((RcvCmd.CmdID & 0xFF) != C1098_CMD_ACK || (RcvCmd.Param1 & 0xFF) != C1098_CMD_SYNC) {
			continue;
		}
		g_CameraState = SYNC_ACK_RECVD;
		DbgPrint("ack done");

		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		bRet = Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD));

		if(RcvCmd.CmdID != C1098_CMD_SYNC) {
			continue;
		}
		g_CameraState = SYNC_RECVD;
		DbgPrint("sync recv");

		bRet = Send_Cmd (C1098_CMD_ACK, C1098_CMD_SYNC, 0x00, 0x00, 0x00);	 // send ACK command
		g_CameraState = SYNC_DONE;
		DbgPrint("send sync ack");
		break;
	}
	MyWait(0xFFFFF);
	return bRet;
}

unsigned char Send_SetPackageSize_Cmd (unsigned short size)
{
	unsigned char bRet = C1098_FALSE;
	
	if (Send_Cmd (C1098_CMD_SETPACSIZE, 0x08, (unsigned char)(size & 0xff),
		(unsigned char)((size & 0xff00) >> 8 ), 0x00) == C1098_TRUE) {
		g_CameraState = SET_PKG_SIZE_SEND;
		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		if (Recv_Cmd((char*)&RcvCmd, sizeof(comedia_C1098_CMD)) == C1098_TRUE) {
			if (RcvCmd.CmdID == C1098_CMD_ACK && RcvCmd.Param1 == C1098_CMD_SETPACSIZE) {
				bRet = C1098_TRUE;
				g_CameraState = SET_PKG_SIZE_ACK_REDVD;
				DbgPrint("SET_PKG_SIZE_ACK_REDVD");
			}
		}
	}
	return bRet;
}


#define DEFAULET_PACKAGE_SIZE ((unsigned short)512)

unsigned char Send_Initial_Cmd (unsigned char resolution)
{
	unsigned char bRet = C1098_FALSE;
	
	if (Send_Cmd (C1098_CMD_INIT, 0x07, 0x07, 0x00, resolution) == C1098_TRUE) {
		g_CameraState = INIT_SEND;

		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		if (Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD)) == C1098_TRUE) 	{
			if (RcvCmd.CmdID == C1098_CMD_ACK && RcvCmd.Param1 == C1098_CMD_INIT) {
				bRet = C1098_TRUE;
				g_CameraState = INIT_ACK_RECVD;
			}
		}
	}
	return bRet;
}


int SendInitCmd(void)
{
	unsigned char	resolution = C1098_RESOLUTION_640x480;
	long			lValue = 0;
	unsigned char	bRet = C1098_FALSE;

	ImageInitial = 0;

	DbgPrint("send Send_SYNC_Cmd");
	// always sync
	bRet = Send_SYNC_Cmd();
	if (bRet == C1098_FALSE) {
		return bRet;
	}
	DbgPrint("Send_SYNC_Cmd done!");


	bRet = Send_Initial_Cmd(resolution);
	if (bRet != C1098_TRUE) {
		DbgPrint("Send_Initial_Cmd fail");
		ImageInitial = 0;
		return C1098_FALSE;
	}
	DbgPrint("Send_Initial_Cmd done!");

	bRet = Send_SetPackageSize_Cmd(DEFAULET_PACKAGE_SIZE);
	if (bRet != C1098_TRUE) {
		DbgPrint("Send_SetPackageSize_Cmd fail");
		ImageInitial = 0;
		return C1098_FALSE;
	}
	DbgPrint("Send_SetPackageSize_Cmd done!");

	ImageInitial = 2;
	Image_resolution = C1098_RESOLUTION_640x480;
	Image_picttype = C1098_PICTTYPE_SNAPSHOT;
	Image_snaptype = C1098_SNAPTYPE_COMPRESS;
	Image_packsize = 0x200;
	
	return bRet;
}

unsigned char Send_Snapshot_Cmd (unsigned char type)
{
	unsigned char bRet = C1098_FALSE;
	int		iState = 0;

	iState = GetCameraState();
	if (iState < DATA_RECVD)
		MyWait(0xFFFFF);

	DbgPrint("send snapshot command");
	if (Send_Cmd(C1098_CMD_SNAP, type, 0x00, 0x00, 0x00) == C1098_TRUE) {
		g_CameraState = SNAPSHOT_SEND;
		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		if (Recv_Cmd((char*)&RcvCmd, sizeof(comedia_C1098_CMD)) == C1098_TRUE) {
			if (RcvCmd.CmdID == C1098_CMD_ACK && RcvCmd.Param1 == C1098_CMD_SNAP) {
				DbgPrint("send snapshot command");
				MyWait(0xFFFFF);
				bRet = C1098_TRUE;
				g_CameraState = SNAPSHOT_ACK_REDVD;
			}
		}
	}
	return bRet;
}


unsigned char Send_GetPicture_Cmd (unsigned char picttype)
{
	unsigned char	bRet = C1098_FALSE;
	int		i;

	// VAJɎʐ^擾NGXg𑗐M
	bRet = Send_Cmd (C1098_CMD_GETPIC, picttype, 0x00, 0x00, 0x00);
	if (bRet == C1098_FALSE) {
		return bRet;
	}
		
	// VAJ̉ҋ@
	for (i=0; i<30; i++) {
		memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
		bRet = Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD));
		if (bRet == C1098_TRUE) {
			break;	
		}
		MyWait(0xFFFF);
	}

	if (bRet == C1098_FALSE) {
		return bRet;
	}

	// 擾ʐ^̃f[^TCY擾
	if (RcvCmd.CmdID == C1098_CMD_ACK && RcvCmd.Param1 == C1098_CMD_GETPIC) {
		for (i=0; i<30; i++) {
			memset(&RcvCmd, 0, sizeof(comedia_C1098_CMD));
			bRet = Recv_Cmd ((char*)&RcvCmd, sizeof(comedia_C1098_CMD));
			if (bRet == C1098_TRUE) {
				break;
			}
			MyWait(0xFFFF);
		}
		// ԐMʐ^̃TCYf[^`FbN
		if (bRet == C1098_TRUE && RcvCmd.CmdID == C1098_CMD_DATA && RcvCmd.Param1 == picttype) {
			// ʐ^̃f[^TCŶŃTCYۑ
			imatge_data_length = (RcvCmd.Param2 | RcvCmd.Param3 << 8 | RcvCmd.Param4 << 16);
			imatge_data_count = 0;		// f[^JEg
		}
	}
	return bRet;
}


unsigned char StartSnapshot(void)
{
	unsigned char bRet = C1098_FALSE;
	
	// ʐ^BeR}h𑗐M
	bRet = Send_Snapshot_Cmd (C1098_SNAPTYPE_COMPRESS);
	if (bRet != C1098_TRUE) {
		return bRet;
	}
	// ʐ^擾R}h𑗐M
	bRet = Send_GetPicture_Cmd (C1098_PICTTYPE_SNAPSHOT);

	return bRet;
}

//
// read recv image data
//
unsigned short ReadImageData(unsigned char* buf, unsigned short size)
{
	unsigned char	bRet = C1098_FALSE;
	unsigned short	wRecvSize = 0;
	
	memset(buf, 0, size);
	memset(imagedate, 0, 512);

	bRet = Send_Cmd (C1098_CMD_ACK, 0x00, 0x00, 0x00ff & imatge_data_count, (0xff00 & imatge_data_count) >> 8);
	if (bRet == C1098_TRUE) {
		// R}hwb_[܂񂾃f[^̓ǂݏo
		bRet = Recv_Cmd ((char*)&imagedate[0], size + 6);
		if (bRet == C1098_TRUE) {
			g_CameraState = DATA_RECVD;
			// 摜̃f[^TCY擾
			wRecvSize = imagedate[2] | ( imagedate[3] << 8 );
			// f[^Rs[
			memcpy(buf, &imagedate[4], wRecvSize);
		}
	}
	return wRecvSize;
}

unsigned char EndReadData(void)
{
	unsigned char bRet = C1098_FALSE;
	
	// ʐ^̎MƂԐM
	bRet = Send_Cmd (C1098_CMD_ACK, 0x00, 0x00, 0x00ff & imatge_data_count, (0xff00 & imatge_data_count) >> 8);
	return bRet;
}



/********************************************************************************/


static char ctrl_buf[H8_CTRL_BUFF_SIZE];

static long H8CtrlC1098Init(int sock)
{
	unsigned char	bRet = C1098_FALSE;
	DbgPrint("Enter H8CtrlC1098Init");
	/* request init command */
	bRet = SendInitCmd();
	
	DbgPrint("SendInitCmd done!");
	
	((H8_CONTROL_PACKET *) ctrl_buf)->CommandCode |= H8_RES_MASK;
	((H8_CONTROL_PACKET *) ctrl_buf)->Status = bRet;
	((H8_CONTROL_PACKET *) ctrl_buf)->TotalPacketCount = 1;
	((H8_CONTROL_PACKET *) ctrl_buf)->CurrentPacketCount = 0;
	((H8_CONTROL_PACKET *) ctrl_buf)->TotalDataLen = 0;
	((H8_CONTROL_PACKET *) ctrl_buf)->DataType = 0;
	((H8_CONTROL_PACKET *) ctrl_buf)->CurrentDataLen = 0;

	//DbgPrint("tcp_write 1\n");
	tcp_write(sock, ctrl_buf, sizeof(H8_CONTROL_PACKET));
	//DbgPrint("tcp_write 2\n");
	return 0;
}


static long H8CtrlC1098GetPicture(int sock)
{
	int				iRet = 0;
	int				iInit = 0;
	unsigned long	lRemainBytes = 0, 
					lSendSize = 0;
	unsigned short	wReadSize = 0;
	unsigned char	*buf_tx = NULL;
	
	iInit = GetInitState();
	if ( iInit == 2) {
		// send snapshot command
		DbgPrint("Enter H8CtrlC1098GetPicture");
		iRet = StartSnapshot();
		if (iRet) {
			// ǂݏoʐ^̃f[^TCYZbg
			lRemainBytes = GetTotalImageSize();	
			((H8_CONTROL_PACKET *) ctrl_buf)->CommandCode |= H8_RES_MASK;
			((H8_CONTROL_PACKET *) ctrl_buf)->Status = iRet;
			((H8_CONTROL_PACKET *) ctrl_buf)->TotalPacketCount = 1;
			((H8_CONTROL_PACKET *) ctrl_buf)->CurrentPacketCount = 0;
			((H8_CONTROL_PACKET *) ctrl_buf)->TotalDataLen = lRemainBytes;
			((H8_CONTROL_PACKET *) ctrl_buf)->DataType = 0;
			((H8_CONTROL_PACKET *) ctrl_buf)->CurrentDataLen = lRemainBytes;
			
			tcp_write(sock, ctrl_buf, sizeof(H8_CONTROL_PACKET));
			// ʐ^𑗐Mobt@m
			buf_tx = (unsigned char *)malloc(2048);
			
			if (buf_tx == NULL) {
				return 0;
			}

			lSendSize = 0;
			while (1) {
				// R}h̃wb_[TCYTCYǂݏoTCYƂ
				wReadSize = 512 - 6;
				// c̃f[^TCYǂݍ݃TCY菬`FbN
				if (lRemainBytes <= wReadSize) {
					// TCŶœǂݍ݃oCgύX
					wReadSize = (unsigned short)lRemainBytes;
				}
				// ʐ^̃f[^ǂݏo
				wReadSize = ReadImageData(&buf_tx[lSendSize], wReadSize);
				IncrementDataCount();
				// ǂݏoc̃oCgXV
				lRemainBytes -= wReadSize;
				// Mf[^TCYXV
				lSendSize += wReadSize;

				// 퓮̏ꍇA506oCgPʂœǂݏôŁA
				// Mobt@ɗ߂2024oCg܂łLOOP
				if (lSendSize == 506 || lSendSize == 506*2 || lSendSize == 506*3) {
					continue;
				}
				// 摜f[^𑗐M
				tcp_write(sock, (char*)&buf_tx[0], lSendSize);
				lSendSize = 0;
				if (lRemainBytes == 0)
					break;
			}
			EndReadData();

			iRet = H8_SUCCESS;
			//DbgPrint("H8CtrlC1098GetPicture: read done!!\n");
			if (buf_tx != NULL)
				free(buf_tx);
		}
	} else {
		//write_mode(ASCII);
		//write_string("H8CtrlC1098GetPicture: need init!!\n");
		((H8_CONTROL_PACKET *) ctrl_buf)->CommandCode |= H8_RES_MASK;
		((H8_CONTROL_PACKET *) ctrl_buf)->Status = H8_FAIL;
		((H8_CONTROL_PACKET *) ctrl_buf)->TotalPacketCount = 1;
		((H8_CONTROL_PACKET *) ctrl_buf)->CurrentPacketCount = 0;
		((H8_CONTROL_PACKET *) ctrl_buf)->TotalDataLen = lRemainBytes;
		((H8_CONTROL_PACKET *) ctrl_buf)->DataType = 0;
		((H8_CONTROL_PACKET *) ctrl_buf)->CurrentDataLen = 0;
		tcp_write(sock, ctrl_buf, sizeof(H8_CONTROL_PACKET));
	}	
	return 0;
}


static int	iBlockFlag = 0;
static int	g_Cnt = 0;

static int g_command = 0;
static int g_socket;

int CommandeRecv(int flag, int sock, struct sockaddr_in *addr) {
	char buffer[256];
	char com[8], name[32], ver[16], *ptr1, *ptr2;
	int  len;
	unsigned char	*p;
	long rcvd;

	rcvd = 0;

	switch(flag) {
		case TCP_BEGIN:
			//write_string("connect\n");
			break;

		case TCP_DATA:
			
			if (iBlockFlag == 1) {
				//write_string("Now working ...\n");
				return -1;
			}
			
			//write_string("CommandeRecv\n");
			
			/* r */
			iBlockFlag = 1;
			len = 0;
				/* Mf[^ǂݏo܂ */
				len = tcp_read(buffer, 255);
				
				if (len == -1) {
					return -1;
				}
					
				if (len > 0) {
					if (rcvd + len > H8_CTRL_BUFF_SIZE) {
						/* error */
						//write_mode(ASCII);
						//write_string("invalid packet length:");
						
						//*p = (unsigned char)len;
						//write_mode(BINARY);
						//sys_write(p, 1);		/* pPbg\ */
						
						//write_mode(ASCII);
						//write_string(" / ");
						//*p = (unsigned char)H8_CTRL_BUFF_SIZE;
						
						//write_mode(BINARY);
						//sys_write(p, 1);		/* pPbg\ */
						
						//write_mode(ASCII);
						//write_string("\n");
					}
					
					/* Mf[^Rs[ */
					memcpy(ctrl_buf + rcvd, buffer, len);
					rcvd += len;

#if DEBUG_COMMAND				
					/* dump command code */
					write_mode(ASCII);
					write_string("CommandCode:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->CommandCode;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");
					

					/* TotalPacketCount */
					write_mode(ASCII);
					write_string("TotalPacketCount:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->CurrentPacketCount;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");
					
					/* CurrentPacketCount */
					write_mode(ASCII);
					write_string("CurrentPacketCount:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->CurrentPacketCount;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");
					
					/* TotalDataLen */
					write_mode(ASCII);
					write_string("TotalDataLen:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->TotalDataLen;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");
					
					/* DataType */
					write_mode(ASCII);
					write_string("DataType:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->DataType;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");

					/* DataType */
					write_mode(ASCII);
					write_string("CurrentDataLen:");
					
					*p = (unsigned char)((H8_CONTROL_PACKET *) ctrl_buf)->CurrentDataLen;
					write_mode(BINARY);
					sys_write(p, 1);
					write_mode(ASCII);
					write_string("\n");
#endif

					if (rcvd >= sizeof(H8_CONTROL_PACKET)) {
						//set
						set_tcpaddr(addr, CTRL_PORT);
						
						switch (((H8_CONTROL_PACKET *) ctrl_buf)->CommandCode) {
							case C1098_INIT:
								g_command = C1098_INIT;
								//write_mode(ASCII);
								//write_string("C1098_INIT\n");
								rcvd = H8CtrlC1098Init(sock);
								break;
							case C1098_GET_PICTURE:
								//write_mode(ASCII);
								//write_string("C1098_GET_PICTURE\n");
								rcvd = H8CtrlC1098GetPicture(sock);
								break;
							case C1098_SNAPSHOT:
								break;
							case H8_RESET:
								break;
							default:
								//write_mode(ASCII);
								//write_string("unknown command!!\n");
								break;
						}
					}
				}
				/*
			} while (len > 0);
				*/
			iBlockFlag = 0;
			break;
		
		case TCP_CLOSE:
			//write_string("disconnect\n");
			tcpclose(sock);
			break;
	}
	return 0;

}

int tcp(unsigned short port, struct sockaddr_in *addr, unsigned char flag, int seq) {
	int sock;

	/* M|[gԍ𔻕 */
	if(port != CTRL_PORT) {
		return -1;
	}
	
	/* ZbgtȌꍇ͐ؒf */
	if(flag & RST) {
		return -1;
	}
	
	/* ڑv̏ꍇؒf */
	if(!(flag & SYN)) {
		return -1;
	}
	
	/* R}hʏɎMpPbgn */
	g_socket = tcpsocket(seq, CommandeRecv);
	//write_string("tcp connect done!!\n");
	
	return 0;
}

int main(void) 
{	
	long rcvd;
	ether_init(0x200000);
	if(get_ether_status() == 0) {
		return -1;
	}
	ip_setup(IPADDR(192,168,0,145), IPADDR(255,255,255,0));
	tcp_regport(tcp);

	/* VA|[gw */
	//sio_change(1);
	sio_change(0);
	
	/* VA|[g */	
	sio_speed(14400, FREQ);
	DbgPrint("start");
	
	SetBaudrate(14400);

	/* loop */
	while(1) {
		if (g_command == C1098_INIT) {
			//rcvd = H8CtrlC1098Init(g_socket);
			g_command = 0;
		}
	}
}






